/* TEXTURE.H - Programmed by Jonathan Hobson */
/* 100404: Maarten Hofman: Stores the name, and allows it to read it */
/* 040704: Maarten Hofman: Added transparancy indicator */
/* 090704: Maarten Hofman: Added animation */

#ifndef __TEXTURE_H__
#define __TEXTURE_H__

#include <sps2lib.h>
#include <sps2tags.h>
#include <assert.h>
#include "ps2defines.h"

enum TIM2_gattr_type {
	TIM2_NONE = 0,			// Used by ClutType when there is no CLUT data
	TIM2_RGB16,				// 16-bit color (used with both ClutType and ImageType)
	TIM2_RGB24,				// 24-bit color (used only with ImageType)
	TIM2_RGB32,				// 32-bit color (used with both ClutType and ImageType)
	TIM2_IDTEX4,			// 16-color texture (used only with ImageType)
	TIM2_IDTEX8				// 16-color texture (used only with ImageType)
};

// TIM2 file header
typedef struct {
	uint8  FileId[4];		// File ID ("T", "I", "M", "2"  or "C", "L", "T", "2")
	uint8  FormatVersion;	// File format version
	uint8  FormatId;		// Format ID
	uint16 Pictures;		// Number of picture data parts
	uint8  pad[8];			// 16-byte alignment
} TIM2_FILEHEADER;

// TIM2 picture header
typedef struct {
	uint32 TotalSize;		// Total size in bytes of all picture data
	uint32 ClutSize;		// Size in bytes of CLUT data part
	uint32 ImageSize;		// Size in bytes of image data part
	uint16 HeaderSize;		// Image header size
	uint16 ClutColors;		// CLUT size (number of colors in CLUT part)
	uint8  PictFormat;		// Image ID
	uint8  MipMapTextures;	// Number of MIPMAPs
	uint8  ClutType;		// CLUT part type
	uint8  ImageType;		// Image part type
	uint16 ImageWidth;		// Image width (not bit count)
	uint16 ImageHeight;		// Image height (not bit count)
	uint64 GsTex0;			// TEX0
	uint64 GsTex1;			// TEX1
	uint32 GsTexaFbaPabe;	// Combination of TEXA, FBA, and PABE bits
	uint32 GsTexClut;		// TEXCLUT (low-order 32 bits directly)
} TIM2_PICTUREHEADER;

// TIM2 MIPMAP header
typedef struct {
	uint64 GsMiptbp1;		// MIPTBP1 (64 bits directly)
	uint64 GsMiptbp2;		// MIPTBP2  (64 bits directly)
	uint32 *MMImageSize;	// Size in bytes of image for MIPMAP number [?]
} TIM2_MIPMAPHEADER;

// TIM2 extended header
typedef struct {
	uint8  ExHeaderId[4];	// Extended comment header  ID("e", "X", "t", "\x00")
	uint32 UserSpaceSize;	// User space size
	uint32 UserDataSize;	// User data size
	uint32 Reserved;		// Reserved
} TIM2_EXHEADER;

int Tim2CheckFileHeader(void *p);
TIM2_PICTUREHEADER *Tim2GetPictureHeader(void *p, int imgno);

class CTexture
{
	public:
		CTexture();
		~CTexture();

		bool LoadImage(const char * strFilename, bool bRedAsAlpha = false);
		void Upload(uint32 uGSPage);
		void Select();
                int has_name(char *);
                bool is_trans();
                void set_trans();
                void set_next(CTexture *how,int what);
                CTexture *get_next();
                void set_name(char *how);
	protected:
		char name[32];
		int m_iTexAddr;
		int m_iClutAddr;

		uint64 m_iTex0;

		int m_iDataSize;

		unsigned char m_pOriginalPalette[256*4];
		unsigned int m_pPalette32[256];

		int m_iImageOriginalDepth, m_iImageDepth;
		int m_iPalBytesPerPixel;

		int m_iWidth, m_iHeight;
		int m_l2Width, m_l2Height;
                bool trans;
                CTexture *next;
                int timing,speed;
};

#endif
